SystemOrganization addCategory: #'GT-Spotter-EventRecorder'!
SystemOrganization addCategory: 'GT-Spotter-EventRecorder-Tests'!

Object subclass: #GTSpotterEventRecorder
	instanceVariableNames: 'recordedData mutex process deliveryDelay deliveryQueue shouldDeliver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder'!

GTSpotterEventRecorder class
	instanceVariableNames: 'instance'!

!GTSpotterEventRecorder commentStamp: 'AndreiChis 3/2/2015 15:02' prior: 0!
A GTSpotterEventRecorder provides the main functionality for recording and sending usage data in Spotter. The collection and delivery works as follows:

- the recorder registers with Spotter every time Spotter is opened ;
- as long as the setting #sendUsageData is set to false all events received by the recorder are discarded;
- the first time an event is send to the recorder and the setting#sendUsageData is set to true a delivery process is started
- as long as the setting #sendUsageData is set to true all interesting events received by the recorder are stored
- at certain intervals the delivery process collects all data, compressed it, and places a payload to a delivery queue.
- the delivery process takes the next paylod from the queue and attepts to send it to a server; on sucsess the payload is removed from the queue.
- if the setting #sendUsageData is set to false, events are no longer collected and all recorded data is removed next time the delivery process resumes.

The delivery process is not stopped by default after creation. The class size method #cleanUp stops the delivery process and removes all data that still awaits delivery.

Instance Variables
	deliveryDelay:		<Object>
	deliveryQueue:		<Object>
	mutex:		<Object>
	process:		<Object>
	recordedData:		<Object>
	shouldDeliver:		<Object>

deliveryDelay
	- xxxxx

deliveryQueue
	- xxxxx

mutex
	- xxxxx

process
	- xxxxx

recordedData
	- xxxxx

shouldDeliver
	- xxxxx
!

GTSpotterEventRecorder class
	instanceVariableNames: 'instance'!

!GTSpotterEventRecorder class methodsFor: 'accessing' stamp: 'JurajKubelka 2/28/2015 00:45'!
cleanUp
	"Stop and remove the process for sending usage data. Delete all recorder that from the queue."
	
	instance ifNotNil: [ 
		instance stopDelivering ifNotNil: [ :aProcess | 
			[ aProcess terminate ] on: Error do: [ "ignore error" ] ].
		instance := nil ]
! !

!GTSpotterEventRecorder class methodsFor: 'accessing' stamp: 'AndreiChis 2/20/2015 10:36'!
instance

	^ instance ifNil: [ instance := self new ]! !

!GTSpotterEventRecorder class methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 15:43'!
materialize: aByteArray
	| materializer stream gzstream materialization |
	stream := aByteArray readStream. 
	gzstream := GZipReadStream on: stream.
	materializer := FLMaterializer newDefault.
	materialization := materializer materializeFrom: gzstream.
	gzstream close.
	stream close.
	^ materialization root.! !

!GTSpotterEventRecorder class methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 17:31'!
materializeObjectsFromDirectory: aFileReference
	"self materializeObjectsFromDirectory: './gt/events/' asFileReference"
	^ aFileReference allFiles collect: [ :eachFile |
		self materializeObjectsFromFile: eachFile ]  ! !

!GTSpotterEventRecorder class methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 17:37'!
materializeObjectsFromFile: aFileReference 
	"self materializeObjectsFromFile: './gt/events/' asFileReference allFiles any"
	^ self materialize: (aFileReference binaryReadStreamDo: #contents)! !

!GTSpotterEventRecorder class methodsFor: 'accessing' stamp: 'AndreiChis 2/20/2015 10:43'!
reset
	instance := nil! !

!GTSpotterEventRecorder class methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 17:43'!
serialize: anObject
	| serializer stream gzstream |
	stream := ByteArray new writeStream binary. 
	gzstream := GZipWriteStream on: stream.
	serializer := FLSerializer newDefault.
	serializer at: #recorderVersion putAdditionalObject: self version.
	serializer stream: gzstream.
	serializer serialize: anObject.
	gzstream close.
	stream close.
	^ stream contents.! !

!GTSpotterEventRecorder class methodsFor: 'accessing' stamp: 'JurajKubelka 2/28/2015 17:41'!
version
	^ 1! !

!GTSpotterEventRecorder methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 17:42'!
announcementFrom: oldRecordedData
	^ GTSpotterRecorderDataAnnouncement new
			data: (self serialize: oldRecordedData);
			timestamp: DateAndTime now;
			imageHash: SmalltalkImage current imageName hash;
			sessionCreationTime: SmalltalkImage current session creationTime;
			imageVersion: SystemVersion current version;
			latestUpdate: SystemVersion current highestUpdate;
			computerUUID: GTSpotterEventRecorderSettings uniqueInstance computerUUID;
			eventRecorderVersion: GTSpotterEventRecorder version;
			yourself! !

!GTSpotterEventRecorder methodsFor: 'events' stamp: 'AndreiChis 2/28/2015 11:57'!
buildAndStoreEventFrom: anAnnouncement withType: aClass

	self storeEvent: (aClass fromAnnouncement: anAnnouncement)

	
 	
		
! !

!GTSpotterEventRecorder methodsFor: 'testing' stamp: 'JurajKubelka 2/26/2015 20:05'!
canSendUsageData
	^ GTSpotterEventRecorderSettings uniqueInstance sendUsageData! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 2/26/2015 14:36'!
deliverAndSignalProgress: aBoolean
	self 
		prepareDeliveryData;
		deliverDataAndSignalProgress: aBoolean.
! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 2/26/2015 17:42'!
deliverData: aByteArray signalProgress: aBoolean
	^ (ZnClient new
			beOneShot;
			timeout: 5;
			signalProgress: aBoolean;
			numberOfRetries: 0;
			url: self remoteURL;
			enforceHttpSuccess: true;
			ifFail: [ :exception | deliveryQueue nextPut: aByteArray ];
			entity: (ZnByteArrayEntity bytes: aByteArray);
			put;
			response)! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 2/26/2015 14:37'!
deliverDataAndSignalProgress: aBoolean
	deliveryQueue flush: [ :eachByteArray |
		self deliverData: eachByteArray signalProgress: aBoolean ]! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 2/28/2015 17:45'!
deliverNow
	"for testing purpose"
	deliveryDelay ifNotNil: [ :aDelay | aDelay delaySemaphore signal. aDelay unschedule. ]! !

!GTSpotterEventRecorder methodsFor: 'accessing' stamp: 'AndreiChis 2/24/2015 22:01'!
eventClassNameFor: aSpotterAnnouncementClass

	^  'GTSpotterRecorder', 
		(aSpotterAnnouncementClass name 
			copyFrom: 'GTSpotter' size + 1
			to: aSpotterAnnouncementClass name size), 
		'Event'.
 	
		
! !

!GTSpotterEventRecorder methodsFor: 'initialization' stamp: 'AndreiChis 2/27/2015 23:09'!
initialize
	super initialize.
	
	recordedData := WaitfreeQueue new.
	deliveryQueue := WaitfreeQueue new.
	mutex := Mutex new.! !

!GTSpotterEventRecorder methodsFor: 'initialization' stamp: 'JurajKubelka 2/26/2015 14:38'!
initializeDeliveryProcess
	process := [ [ self shouldDeliver ] whileTrue: [ 
						self deliverAndSignalProgress: false.
						deliveryDelay := self standardDuration asDelay.
						deliveryDelay wait.
						deliveryDelay := nil. ] ] 
		forkAt: Processor userBackgroundPriority 
		named: self className, ' regular delivery process'! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 2/26/2015 14:37'!
lastDataDelivery
	UIManager default informUserDuring: [ :bar |
	  	[ self deliverAndSignalProgress: true ]
	    on: HTTPProgress 
	    do: [ :progress |  
			progress isEmpty ifFalse: [ bar current: progress percentage ].
	      progress resume ] ]
! !

!GTSpotterEventRecorder methodsFor: 'initialization' stamp: 'JurajKubelka 2/25/2015 17:51'!
linkToSpotter: aSpotterModel

	aSpotterModel announcer weak when: Announcement send: #recordAccouncement: to: self! !

!GTSpotterEventRecorder methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 15:43'!
materialize: aByteArray
	^ self class materialize: aByteArray! !

!GTSpotterEventRecorder methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 15:42'!
materializedAnnouncementFrom: aByteArray 
	^ self materialize: aByteArray! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 2/28/2015 17:05'!
prepareDeliveryData
	| toBeDeliveredData toBeDeliveredAnnouncement |
	GTSpotterEventRecorderSettings uniqueInstance sendUsageData ifFalse: [ 
		recordedData removeAll.
		deliveryQueue removeAll.
		^ self ].
	recordedData isEmpty ifTrue: [ ^ self ].
	toBeDeliveredData := OrderedCollection new.
	recordedData flush: [ :eachRecord | toBeDeliveredData addLast: eachRecord ].
	toBeDeliveredAnnouncement := self serializedAnnouncementFrom: toBeDeliveredData.
	deliveryQueue nextPut: toBeDeliveredAnnouncement.! !

!GTSpotterEventRecorder methodsFor: 'events' stamp: 'AndreiChis 2/28/2015 18:17'!
recordAccouncement: anAnnouncement

	self canSendUsageData ifFalse: [ ^ self ].

	"recorderEventClassName := self eventClassNameFor: anAnnouncement class.
	(Smalltalk hasClassNamed: recorderEventClassName) ifFalse: [ ^ false ].
	self storeEvent: (recorderEventClassName asClass fromAnnouncement: anAnnouncement)"
	
	anAnnouncement class == GTSpotterActOn ifTrue: [ ^self buildAndStoreEventFrom: anAnnouncement withType: GTSpotterRecorderActOnEvent ].
	
	anAnnouncement class == GTSpotterCategorySelected ifTrue: [ ^self buildAndStoreEventFrom: anAnnouncement withType: GTSpotterRecorderCategorySelectedEvent ].
	
	anAnnouncement class == GTSpotterContextChanged ifTrue: [ ^self buildAndStoreEventFrom: anAnnouncement withType: GTSpotterRecorderContextChangedEvent ].
	
	anAnnouncement class == GTSpotterDiveInCategory ifTrue: [ ^self buildAndStoreEventFrom: anAnnouncement withType: GTSpotterRecorderDiveInCategoryEvent ].
	
	anAnnouncement class == GTSpotterDiveInElement ifTrue: [ ^self buildAndStoreEventFrom: anAnnouncement withType: GTSpotterRecorderDiveInElementEvent ].
	
	anAnnouncement class == GTSpotterDiveOut ifTrue: [ ^self buildAndStoreEventFrom: anAnnouncement withType: GTSpotterRecorderDiveOutEvent ].
	
	anAnnouncement class == GTSpotterExitAnnouncement ifTrue: [ ^self buildAndStoreEventFrom: anAnnouncement withType: GTSpotterRecorderExitAnnouncementEvent ].
	
	anAnnouncement class == GTSpotterHidePreview ifTrue: [ ^self buildAndStoreEventFrom: anAnnouncement withType: GTSpotterRecorderHidePreviewEvent ].
	
	anAnnouncement class == GTSpotterSelectedCandidateChanged ifTrue: [ ^self buildAndStoreEventFrom: anAnnouncement withType: GTSpotterRecorderSelectedCandidateChangedEvent ].
	
	anAnnouncement class == GTSpotterShowPreview ifTrue: [ ^self buildAndStoreEventFrom: anAnnouncement withType: GTSpotterRecorderShowPreviewEvent ].
	
	anAnnouncement class == GTSpotterSearchFinished ifTrue: [ ^self buildAndStoreEventFrom: anAnnouncement withType: GTSpotterRecorderSearchFinishedEvent ].
	

	
 	
		
! !

!GTSpotterEventRecorder methodsFor: 'events' stamp: 'JurajKubelka 2/26/2015 20:06'!
recordSpotterOpenEvent

	self canSendUsageData ifFalse: [ ^ self ].

	self storeEvent: GTSpotterRecorderOpenEvent new! !

!GTSpotterEventRecorder methodsFor: 'accessing' stamp: 'JurajKubelka 2/25/2015 13:17'!
remoteURL
	^ GTSpotterEventRecorderSettings url! !

!GTSpotterEventRecorder methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 15:45'!
serialize: anObject
	^ self class serialize: anObject! !

!GTSpotterEventRecorder methodsFor: 'converting' stamp: 'JurajKubelka 2/26/2015 14:16'!
serializedAnnouncementFrom: oldRecordedData
	^ self serialize: (self announcementFrom: oldRecordedData)! !

!GTSpotterEventRecorder methodsFor: 'accessing' stamp: 'JurajKubelka 2/25/2015 12:18'!
shouldDeliver
	^ shouldDeliver ifNil: [ shouldDeliver := true ]! !

!GTSpotterEventRecorder methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 18:32'!
standardDuration
	^ 20 minutes! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 2/27/2015 19:42'!
stopDelivering
	^ mutex critical: [ | oldProcess |
		shouldDeliver := false.
		deliveryDelay ifNotNil: [ :aDelay | aDelay delaySemaphore signal ].
		oldProcess := process.
		process := nil.
		oldProcess ]! !

!GTSpotterEventRecorder methodsFor: 'events' stamp: 'AndreiChis 2/27/2015 23:15'!
storeEvent: anEvent
	recordedData nextPut: anEvent.
	mutex critical: [ 
		(process isNil or: [ process isTerminated ]) ifTrue: [ self initializeDeliveryProcess ] ]! !

Object subclass: #GTSpotterEventRecorderSettings
	instanceVariableNames: 'computerUUID sendUsageData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder'!

GTSpotterEventRecorderSettings class
	instanceVariableNames: 'url uniqueInstance'!

!GTSpotterEventRecorderSettings commentStamp: 'AndreiChis 3/2/2015 15:21' prior: 0!
A GTSpotterEventRecorderSettings manages  user preferences related to  sending usage data. This works as follows:  

1. First opening of an image with the recorder
- the first time an image containing this class is opened (or  this class is loaded in an image) a settings file is created in the settings folder of the current user (FileLocator preferences asFileReference).
- a UUID is generated and store d in the setting file to identfy the current computer

2. Opening an image after a settings file was created
- if this image does not already have a setting stored in an image or has a setting with a different UUID then the one of the local computer, any setting present in the image is removed.
- as a second step, if there is a default setting present in the settings file that setting is loaded and stored in the image.

3. Changing the default settings
- the default setting can only be changed from the Agreement UI accessible from the Spotter settings.

This mechanism has the following implications:
- as long as there is no default setting stored a setting has to be stored in every image manually
- if there is a default setting stored whenever opening a new image without an explicit setting the default setting becames is saved in the image.

Hence, changing the default setting later on will not change the existing setting in any existing images. It will only apply to new images.
Also copying an image from one computer to another resets it's settings to the default settings of the destination computer.

Instance Variables
	computerUUID:		<Object>
	sendSearchQuery:		<Object>
	sendUsageData:		<Object>
	url:		<Object>
	userAnsweredAgreement:		<Object>

computerUUID
	- xxxxx

sendSearchQuery
	- xxxxx

sendUsageData
	- xxxxx

url
	- xxxxx

userAnsweredAgreement
	- xxxxx
!

GTSpotterEventRecorderSettings class
	instanceVariableNames: 'url uniqueInstance'!

!GTSpotterEventRecorderSettings class methodsFor: 'accessing' stamp: 'AndreiChis 3/1/2015 23:13'!
defaultSaveUserDataPreference
	|dictionary sendUsageDataValue |

	self preferences exists ifFalse: [ ^ nil ].
	[ 
		dictionary := FLMaterializer materializeFromFileNamed: self preferences.
		sendUsageDataValue := dictionary at:#sendUsageData ifAbsent: [ ^ nil ].
		sendUsageDataValue ifNil: [ ^ nil ].
		^ sendUsageDataValue 
	] on: Error do: [ 
		^ nil]! !

!GTSpotterEventRecorderSettings class methodsFor: 'load and saving' stamp: 'JurajKubelka 2/26/2015 18:58'!
deletePreferences
	"self deletePreferences"
	[ self preferences ensureDelete ]
	on: Error do: [ "ignore" ]! !

!GTSpotterEventRecorderSettings class methodsFor: 'reflective operations' stamp: 'JurajKubelka 2/25/2015 14:35'!
doesNotUnderstand: aMessage 
	"delegate all settings to the unique instance"
	^ self uniqueInstance perform: aMessage selector withEnoughArguments: aMessage arguments! !

!GTSpotterEventRecorderSettings class methodsFor: 'class initialization' stamp: 'JurajKubelka 2/27/2015 17:28'!
initialize
	super initialize.
	Smalltalk addToStartUpList: self.
	self loadPreferences.! !

!GTSpotterEventRecorderSettings class methodsFor: 'load and saving' stamp: 'AndreiChis 3/1/2015 11:26'!
loadPreferences
	"self loadPreferences"
	| dictionary newInstance |
	self preferences exists ifFalse: [ 
		"This is a new computer, so we define new computer UUID.
		User still has to agree about sending data if it is not has been done yet."
		^ self savePreferences ].
	[ 
		dictionary := FLMaterializer materializeFromFileNamed: self preferences.
		newInstance := self new dictionary: dictionary.
		"If UUIDs are different, it is likely different user and we should use
		localy stored settings. If the UUIDs are same, we keep settings of the image."
		newInstance computerUUID = self uniqueInstance computerUUID
			ifFalse: [ uniqueInstance := newInstance ]
	] on: Error do: [ 
		"Preferences likely contains a different settings version, so we store the actual one.
		We should keep the preferences as stable as possible."
		self savePreferences ]! !

!GTSpotterEventRecorderSettings class methodsFor: 'origins' stamp: 'JurajKubelka 2/26/2015 18:40'!
preferences
	^ FileLocator preferences / 'org.pharo.gt.spotter.event.recorder.fuel' ! !

!GTSpotterEventRecorderSettings class methodsFor: 'accessing' stamp: 'JurajKubelka 2/25/2015 17:04'!
reset
	"self reset"
	uniqueInstance := nil! !

!GTSpotterEventRecorderSettings class methodsFor: 'load and saving' stamp: 'JurajKubelka 2/28/2015 15:49'!
savePreferences
	"self savePreferences"
	self deletePreferences.
	[ FLSerializer serialize: self uniqueInstance dictionary toFileNamed: self preferences ]
		on: Error do: [ "ignore" ]! !

!GTSpotterEventRecorderSettings class methodsFor: 'settings' stamp: 'JurajKubelka 2/25/2015 14:38'!
settingsOnUrl: aBuilder 
	<systemsettings>
	
	(aBuilder setting: #stringUrl)
		parent: #gt;
		target: self;
		description: 'The url for the Spotter Event Recorder server.' translated;
		label: 'Spotter Event Recorder URL address'
! !

!GTSpotterEventRecorderSettings class methodsFor: 'system startup' stamp: 'JurajKubelka 2/27/2015 17:32'!
startUp: resuming
	"We reset image preferences, because this is likely 
	a newly downloaded image or different user
	and he/she should agree about sending data."
	self preferences exists ifFalse: [ self reset ].
	self loadPreferences.! !

!GTSpotterEventRecorderSettings class methodsFor: 'accessing' stamp: 'JurajKubelka 2/27/2015 16:08'!
stringUrl
	^ self url asString! !

!GTSpotterEventRecorderSettings class methodsFor: 'accessing' stamp: 'JurajKubelka 2/27/2015 16:08'!
stringUrl: aUrlString
	^ self url: aUrlString! !

!GTSpotterEventRecorderSettings class methodsFor: 'accessing' stamp: 'JurajKubelka 2/25/2015 13:45'!
uniqueInstance
	^ uniqueInstance ifNil: [ uniqueInstance := self new ]! !

!GTSpotterEventRecorderSettings class methodsFor: 'accessing' stamp: 'JurajKubelka 2/27/2015 16:08'!
url
	^ url ifNil: [ url := 'http://gc.dcc.uchile.cl.:8080/gt/events' asZnUrl ]! !

!GTSpotterEventRecorderSettings class methodsFor: 'accessing' stamp: 'AndreiChis 3/1/2015 22:05'!
url: aZnUrl
	url := aZnUrl ifNotNil: [ aZnUrl asZnUrl ].
	self flag:'Not clear why save is called in this method'.
	"self save."! !

!GTSpotterEventRecorderSettings methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 18:39'!
computerUUID
	^ computerUUID! !

!GTSpotterEventRecorderSettings methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 15:55'!
dictionary
	| dictionary |
	dictionary := Dictionary new: self class instSize.
	1 to: self class instSize do: [ :index |
		dictionary 
			at: (self class instVarNameForIndex: index)
			put: (self instVarAt: index)
	].
	^ dictionary! !

!GTSpotterEventRecorderSettings methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 16:04'!
dictionary: aCollection 
	aCollection keysAndValuesDo: [ :key :value | | instVarIndex |
		instVarIndex := self class instVarIndexFor: key.
		instVarIndex isZero ifFalse: [ 
			self instVarAt: instVarIndex put: value
		]
	]! !

!GTSpotterEventRecorderSettings methodsFor: 'initialization' stamp: 'JurajKubelka 2/26/2015 18:39'!
initialize
	super initialize.
	computerUUID := UUID new.
	! !

!GTSpotterEventRecorderSettings methodsFor: 'actions' stamp: 'AndreiChis 3/1/2015 22:00'!
persistSettings
	self save.! !

!GTSpotterEventRecorderSettings methodsFor: 'actions' stamp: 'AndreiChis 3/1/2015 22:00'!
save
	self class savePreferences! !

!GTSpotterEventRecorderSettings methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 20:05'!
sendUsageData
	^ sendUsageData ifNil: [ sendUsageData := false ]! !

!GTSpotterEventRecorderSettings methodsFor: 'accessing' stamp: 'AndreiChis 3/1/2015 22:00'!
sendUsageData: aBoolean
	sendUsageData := aBoolean.! !

Object subclass: #GTSpotterRecorderEvent
	instanceVariableNames: 'timestamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder'!

!GTSpotterRecorderEvent commentStamp: 'AndreiChis 2/27/2015 14:31' prior: 0!
GTSpotterRecorderEvent is the root class for representing usage data generated by Spotter. 
!

GTSpotterRecorderEvent subclass: #GTSpotterRecorderActOnEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder'!

!GTSpotterRecorderActOnEvent commentStamp: 'AndreiChis 2/27/2015 14:35' prior: 0!
A GTSpotterRecorderActOnEvent models the action of pressing enter (or double clicking) on an element.
!

GTSpotterRecorderEvent subclass: #GTSpotterRecorderCategorySelectedEvent
	instanceVariableNames: 'categoryLabel elementCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder'!

!GTSpotterRecorderCategorySelectedEvent commentStamp: 'AndreiChis 2/27/2015 14:41' prior: 0!
A GTSpotterRecorderCategorySelectedEvent  models the action of  selecting a category in the search results. This event stores the name of the category and the number of results that were matched by the current query.

Instance Variables
	categoryLabel:		<Object>
	elementCount:		<Object>

categoryLabel
	- xxxxx

elementCount
	- xxxxx
!

!GTSpotterRecorderCategorySelectedEvent methodsFor: 'initialization' stamp: 'AndreiChis 2/24/2015 21:36'!
fromAnnouncement: anAnnouncement

	categoryLabel := anAnnouncement category title.
	elementCount := anAnnouncement category allFilteredCandidates size.! !

GTSpotterRecorderEvent subclass: #GTSpotterRecorderContextChangedEvent
	instanceVariableNames: 'querySize isRefinement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder'!

!GTSpotterRecorderContextChangedEvent commentStamp: 'JurajKubelka 2/27/2015 16:12' prior: 0!
A GTSpotterRecorderContextChangedEvent stores current query written in Spotter input field.

Instance Variables
	text:		<String>

text
	- query writen in Spotter input field
!

!GTSpotterRecorderContextChangedEvent methodsFor: 'initialization' stamp: 'JurajKubelka 2/28/2015 17:17'!
fromAnnouncement: anAnnouncement

	querySize := anAnnouncement currentText size.
	anAnnouncement previousText size == 0 ifTrue: [ ^ self ].
	isRefinement := (anAnnouncement currentText size > anAnnouncement previousText size 
		ifTrue: [ anAnnouncement currentText beginsWith: anAnnouncement previousText]
		ifFalse: [ anAnnouncement previousText beginsWith: anAnnouncement currentText ]).! !

!GTSpotterRecorderContextChangedEvent methodsFor: 'initialization' stamp: 'JurajKubelka 2/28/2015 11:59'!
initialize
	super initialize.
	isRefinement := false.! !

GTSpotterRecorderEvent subclass: #GTSpotterRecorderDiveInCategoryEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder'!

!GTSpotterRecorderDiveInCategoryEvent commentStamp: 'AndreiChis 2/27/2015 14:36' prior: 0!
A GTSpotterRecorderDiveInCategoryEvent  models the action of  diving in a category (list of results).
!

GTSpotterRecorderEvent subclass: #GTSpotterRecorderDiveInElementEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder'!

!GTSpotterRecorderDiveInElementEvent commentStamp: 'AndreiChis 2/27/2015 14:36' prior: 0!
A GTSpotterRecorderDiveInElementEvent models the action of diving in a selected element.
!

GTSpotterRecorderEvent subclass: #GTSpotterRecorderDiveOutEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder'!

!GTSpotterRecorderDiveOutEvent commentStamp: 'AndreiChis 2/27/2015 14:37' prior: 0!
A GTSpotterRecorderDiveOutEvent  models the action of  diving out a step.
!

!GTSpotterRecorderEvent class methodsFor: 'instance creation' stamp: 'AndreiChis 2/24/2015 21:08'!
fromAnnouncement: anAnnouncement

	^ self new fromAnnouncement: anAnnouncement! !

!GTSpotterRecorderEvent methodsFor: 'initialization' stamp: 'AndreiChis 2/24/2015 21:08'!
fromAnnouncement: anAnnouncement

	! !

!GTSpotterRecorderEvent methodsFor: 'initialization' stamp: 'JurajKubelka 2/28/2015 00:47'!
initialize
	super initialize.
	
	timestamp := Time now asDateAndTime asUnixTime ! !

GTSpotterRecorderEvent subclass: #GTSpotterRecorderExitAnnouncementEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder'!

!GTSpotterRecorderExitAnnouncementEvent commentStamp: 'AndreiChis 2/27/2015 14:37' prior: 0!
A GTSpotterRecorderExitAnnouncementEvent  models the action of  exiting Spotter. 

Instance Variables
	timestamp:		<Object>

timestamp
	- xxxxx
!

GTSpotterRecorderEvent subclass: #GTSpotterRecorderHidePreviewEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder'!

!GTSpotterRecorderHidePreviewEvent commentStamp: 'AndreiChis 2/27/2015 14:38' prior: 0!
A GTSpotterRecorderHidePreviewEvent  models the action of explicitly hidding the preview.
!

GTSpotterRecorderEvent subclass: #GTSpotterRecorderOpenEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder'!

!GTSpotterRecorderOpenEvent commentStamp: 'AndreiChis 2/27/2015 14:38' prior: 0!
A GTSpotterRecorderOpenEvent  models the action of  opening Spotter.

Instance Variables
	timestamp:		<Object>

timestamp
	- xxxxx
!

GTSpotterRecorderEvent subclass: #GTSpotterRecorderSearchFinishedEvent
	instanceVariableNames: 'querySize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder'!

!GTSpotterRecorderSearchFinishedEvent commentStamp: 'AndreiChis 2/28/2015 19:06' prior: 0!
A GTSpotterRecorderSearchFinishedEvent is triggered when spotter finished a search and displayed all the results.
!

!GTSpotterRecorderSearchFinishedEvent methodsFor: 'initialization' stamp: 'AndreiChis 2/28/2015 19:01'!
fromAnnouncement: anAnnouncement

	querySize := anAnnouncement query ifNil: [ 0 ]  ifNotNil: [ :aString | aString size]! !

GTSpotterRecorderEvent subclass: #GTSpotterRecorderSelectedCandidateChangedEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder'!

!GTSpotterRecorderSelectedCandidateChangedEvent commentStamp: 'AndreiChis 2/28/2015 19:07' prior: 0!
A GTSpotterRecorderSelectedCandidateChangedEvent models the action of  selecting an element in the search results.
!

GTSpotterRecorderEvent subclass: #GTSpotterRecorderShowPreviewEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder'!

!GTSpotterRecorderShowPreviewEvent commentStamp: 'AndreiChis 2/27/2015 14:39' prior: 0!
A GTSpotterRecorderShowPreviewEvent models the action of explicitly closing the preview of an object.
!

Announcement subclass: #GTSpotterRecorderDataAnnouncement
	instanceVariableNames: 'computerUUID data eventRecorderVersion imageHash imageVersion latestUpdate sessionCreationTime timestamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder'!

!GTSpotterRecorderDataAnnouncement commentStamp: 'AndreiChis 2/27/2015 14:35' prior: 0!
A GTSpotterRecorderDataAnnouncement is an announcement that  groups  all  usage data  that is send over the network. 

Instance Variables
	computerUUID:		<Object>
	data:		<Object>
	imageHash:		<Object>
	imageVersion:		<Object>
	latestUpdate:		<Object>
	sessionCreationTime:		<Object>
	timestamp:		<Object>

computerUUID
	- xxxxx

data
	- xxxxx

imageHash
	- xxxxx

imageVersion
	- xxxxx

latestUpdate
	- xxxxx

sessionCreationTime
	- xxxxx

timestamp
	- xxxxx
!

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 18:27'!
computerUUID
	^ computerUUID! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 18:27'!
computerUUID: anObject
	computerUUID := anObject! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/25/2015 15:12'!
data
	^ data! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/25/2015 15:11'!
data: aByteArray 
	data := aByteArray! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/28/2015 17:42'!
eventRecorderVersion
	^ eventRecorderVersion! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/28/2015 17:42'!
eventRecorderVersion: anObject
	eventRecorderVersion := anObject! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/25/2015 18:08'!
imageHash
	^ imageHash! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/25/2015 18:08'!
imageHash: anObject
	imageHash := anObject! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 18:26'!
imageVersion
	^ imageVersion! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 18:26'!
imageVersion: anObject
	imageVersion := anObject! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 18:26'!
latestUpdate
	^ latestUpdate! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 18:26'!
latestUpdate: anObject
	latestUpdate := anObject! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/28/2015 15:44'!
materializedData
	^ self data ifNotNil: [ :aData | GTSpotterEventRecorder materialize: aData ]! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/25/2015 18:08'!
sessionCreationTime
	^ sessionCreationTime! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/25/2015 18:08'!
sessionCreationTime: anObject
	sessionCreationTime := anObject! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/25/2015 18:09'!
timestamp
	^ timestamp! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/25/2015 18:09'!
timestamp: anObject
	timestamp := anObject! !

TestCase subclass: #GTSpotterEventRecorderTest
	instanceVariableNames: 'recorder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Tests'!

!GTSpotterEventRecorderTest commentStamp: '<historical>' prior: 0!
A GTSpotterEventRecorderTest is a test class for testing the behavior of GTSpotterEventRecorder!

!GTSpotterEventRecorderTest methodsFor: 'as yet unclassified' stamp: 'JurajKubelka 2/28/2015 15:32'!
recordedData
	^ GTSpotterRecorderEvent allSubclasses collect: #new! !

!GTSpotterEventRecorderTest methodsFor: 'as yet unclassified' stamp: 'JurajKubelka 2/28/2015 15:34'!
setUp
	super setUp.
	recorder := GTSpotterEventRecorder new.
	recorder stopDelivering.! !

!GTSpotterEventRecorderTest methodsFor: 'as yet unclassified' stamp: 'JurajKubelka 2/28/2015 15:45'!
testSerializedAnnouncementFrom
	| byteArray announcement |
	byteArray := recorder serializedAnnouncementFrom: self recordedData.
	announcement := recorder materializedAnnouncementFrom: byteArray.
	self assert: announcement class equals: GTSpotterRecorderDataAnnouncement.
	self assert: announcement materializedData isCollection.
	self assert: announcement materializedData size equals: self recordedData size.! !
GTSpotterEventRecorderSettings initialize!
